--[[ LookingForSatchels ]]

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
}

local frame = CreateFrame("Frame", "LookingForSatchelsFrame", UIParent)
local frameEvents = {};
local moving = false



local function initFrame()
  frame:SetPoint(LookingForSatchelsOptions["framePoint"], LookingForSatchelsOptions["frameRelativeTo"], LookingForSatchelsOptions["frameRelativePoint"], LookingForSatchelsOptions["frameOffsetX"], LookingForSatchelsOptions["frameOffsetY"])
  frame:SetFrameStrata("LOW")
  frame:SetSize(50, 12)
  
  frame.bgtexture = frame:CreateTexture(nil, "OVERLAY")
  frame.bgtexture:SetAllPoints(frame)
  frame.bgtexture:SetTexture(0, 0, 0, 1)
  frame.bgtexture:Hide()
  
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    LookingForSatchelsOptions["framePoint"] = point or "LEFT"
    LookingForSatchelsOptions["frameRelativeTo"] = relativeTo or "UIParent"
    LookingForSatchelsOptions["frameRelativePoint"] = relativePoint or "CENTER"
    LookingForSatchelsOptions["frameOffsetX"] = xOfs
    LookingForSatchelsOptions["frameOffsetY"] = yOfs
  end);
  
  --------------------
  --LFG satchel grabber
  --------------------
  frame.LFGsearchActive = false
  frame.LFGsearchActiveAtLogin = true
  frame.LFGtslU = 0
  frame.LFGInterval = 10
  frame.LFGIntervalAtLogin = 10 --seconds, after which initial search at login stops
  frame.LFGsearchLastScanFoundReward = false
  frame.LFG_roles = {1, 0, 0} --Tank, Heal, Damage
  
  frame.LFG_dungeonIDs = {
    --[123] = 1;
    --[234] = 0;
  };
  
  frame.addCurrentLFGId = function(dungeonID)
    if (type(dungeonID) ~= "number") or LFGIsIDHeader(dungeonID) then
      return
    end
    
    frame.LFG_dungeonIDs[dungeonID] = 1
    
    --LFDQueueFrame.type
    --LFRQueueFrame.selectedLFM
    frame.LFGsearchActive = true
    frame.LFGtslU = frame.LFGInterval
    local name = GetLFGDungeonInfo(dungeonID)
    print(format("|cffaaaaffLookingForSatchels: added LFG search for: %s", name))
    
    frame.updateDisplay()
  end
  frame.removeCurrentLFGId = function(dungeonID)
    if (type(dungeonID) ~= "number") or LFGIsIDHeader(dungeonID) then
      return
    end
    
    frame.LFG_dungeonIDs[dungeonID] = 0
    local active = false
    for k, v in pairs(frame.LFG_dungeonIDs) do
      if v >= 1 then
        active = true
        break
      end
    end
    frame.LFGsearchActive = active
    local name = GetLFGDungeonInfo(dungeonID)
    print(format("|cffaaaaffLookingForSatchels: removed LFG search for: %s", name))
    
    frame.updateDisplay()
  end
  
  frame.frameLFGSearchButtons = {
    CreateFrame("Button", nil, LFDQueueFrameTypeDropDown),
    CreateFrame("Button", nil, ScenarioQueueFrameTypeDropDown),
    CreateFrame("Button", nil, RaidFinderQueueFrameSelectionDropDown),
  };
  for i, b in ipairs(frame.frameLFGSearchButtons) do
    --_G["LFS_CombatTime_LFGSearchButton"..i] = b
    
    b:SetPoint("RIGHT", _G[(b:GetParent():GetName()).."Name"], "RIGHT", -60, 0)
    b:SetFrameStrata("DIALOG")
    b:SetWidth(30)
    b:SetHeight(22)
    
    b.status = 0
    b.updateText = function(self)
      if self.status == 0 then
        b:SetText("L+")
      else
        b:SetText("L-")
      end
    end
    b:updateText()
    
    b.frameRef = frame
    b:SetScript("OnClick", function(self, arg1)
      if self.status == 0 then
        self.frameRef.addCurrentLFGId((self:GetParent()).selectedValue)
        self.status = 1
      else
        self.frameRef.removeCurrentLFGId((self:GetParent()).selectedValue)
        self.status = 0
      end
      self:updateText()
    end);
    
    b:SetNormalFontObject("GameFontNormal")
    
    local ntex = b:CreateTexture()
    ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
    ntex:SetTexCoord(0, 0.625, 0, 0.6875)
    ntex:SetAllPoints()	
    b:SetNormalTexture(ntex)

    local htex = b:CreateTexture()
    htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
    htex:SetTexCoord(0, 0.625, 0, 0.6875)
    htex:SetAllPoints()
    b:SetHighlightTexture(htex)

    local ptex = b:CreateTexture()
    ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
    ptex:SetTexCoord(0, 0.625, 0, 0.6875)
    ptex:SetAllPoints()
    b:SetPushedTexture(ptex)
  end
  
  --[ [
  frame.hook_LFGRewardsFrame_UpdateFrame = function(parentFrame, dungeonID, background)
    if (type(dungeonID) == "number") and not LFGIsIDHeader(dungeonID) then
      local i = frame.LFG_dungeonIDs[dungeonID]
      if not i then
        i = 0
        frame.LFG_dungeonIDs[dungeonID] = 0
      end
      for i2, b in ipairs(frame.frameLFGSearchButtons) do
        b.status = i
        b:updateText()
      end
    end
  end
  hooksecurefunc("LFGRewardsFrame_UpdateFrame", frame.hook_LFGRewardsFrame_UpdateFrame)
  --] ]
  
  --------------------
  --frame text
  --------------------
  frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  frame.text:SetPoint("RIGHT")
  frame.text:SetText("L")
  frame.text:SetTextColor(1, 1, 1, 1)
  
  frame.OnUpdate = function(self, elapsed)
    --LFG search at login will be stopped after 10 seconds
    if self.LFGsearchActiveAtLogin then
      self.LFGIntervalAtLogin = self.LFGIntervalAtLogin - elapsed
      if self.LFGIntervalAtLogin <= 0 then
        self.LFGsearchActiveAtLogin = false
      end
    end
    
    self.LFGtslU = self.LFGtslU + elapsed
    if self.LFGsearchActive and (self.LFGtslU >= self.LFGInterval) then
      self.LFGtslU = 0
      RequestLFDPlayerLockInfo()
    end
  end
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  
  frame.updateDisplay = function()
    local t
    if frame.LFGsearchActive then
      if frame.LFGsearchLastScanFoundReward then
        t = "|cff00ff00F|r"
      else
        t = "|cffffff00L|r"
      end
    else
      t = "|cffff0000L|r"
    end
    frame.text:SetText(t)
  end
end


function frameEvents:PLAYER_ENTERING_WORLD(...)
  frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
  
  if not LookingForSatchelsOptions then
    LookingForSatchelsOptions = DefaultO
  end
  initFrame()
  --updateOptionsArray()
  
  
  frame:RegisterEvent("LFG_UPDATE_RANDOM_INFO")
end

function frameEvents:LFG_UPDATE_RANDOM_INFO()
  if frame.LFGsearchActive or frame.LFGsearchActiveAfterLogin then
    local foundReward = false
    for k, v in pairs(frame.LFG_dungeonIDs) do
      if v >= 1 then
        local foundRewardForK = false
        for i = 1, LFG_ROLE_NUM_SHORTAGE_TYPES do
          local eligible, forTank, forHealer, forDamage, itemCount, money, xp = GetLFGRoleShortageRewards(k, i)
          if eligible and (forTank and frame.LFG_roles[1]==1 or forHealer and frame.LFG_roles[2]==1 or forDamage and frame.LFG_roles[3]==1) and (itemCount ~= 0 or money ~= 0 or xp ~= 0) then
            foundReward = true
            foundRewardForK = true
            
            if frame.LFG_dungeonIDs[k] == 1 then
              frame.LFG_dungeonIDs[k] = 2
              
              local name = GetLFGDungeonInfo(k)
              print("|cffaaaaffLFG reward for: "..name)
              local warningColor = {
                ["r"] = 0.67;
                ["g"] = 0.67;
                ["b"] = 1;
              };
              RaidNotice_AddMessage(RaidWarningFrame, "LFG reward for: "..name, warningColor, 10)
            end
            
            if not frame.LFGsearchLastScanFoundReward then
              FlashClientIcon()
              frame.LFGsearchLastScanFoundReward = true
              --return
            end
          end
        end
        if not foundRewardForK then
          frame.LFG_dungeonIDs[k] = 1
        end
      end
    end
    
    if frame.LFGsearchLastScanFoundReward and (not foundReward) then
      frame.LFGsearchLastScanFoundReward = false
    end
    
    frame.updateDisplay()
  end
end

frame:SetScript("OnEvent", function(self, event, ...)
  frameEvents[event](self, ...)
end);
frame:RegisterEvent("PLAYER_ENTERING_WORLD")

--split string containing quoted and not quoted arguments
--input pattern: (\S+|".+")?(\s+(\S+|".+"))*
--example input: [[arg1 "arg2part1 arg2part2" arg3]]
--example output: {"arg1", "arg2part1 arg2part2", "arg3"}
local function mysplit2(inputstr)
  local i, i1, i2, l, ret, retI = 1, 0, 0, inputstr:len(), {}, 1
  --remove leading spaces
  i1, i2 = inputstr:find("^%s+")
  if i1 then
    i = i2 + 1
  end
  
  while i <= l do
    --find end of current arg
    if (inputstr:sub(i, i)) == "\"" then
      --quoted arg, find end quote
      i1, i2 = inputstr:find("\"%s+", i + 1)
      if i1 then
        --spaces after end quote, more args to follow
        ret[retI] = inputstr:sub(i + 1, i1 - 1)
        retI = retI + 1
        i = i2 + 1
      else
        i1, i2 = inputstr:find("\"$", i + 1)
        if i1 then
          --end of msg
          ret[retI] = inputstr:sub(i + 1, i1 - 1)
          return ret
        else
          -- no end quote found, or end quote followed by no-space-charater found, disregard last arg
          return ret
        end
      end
    else
      --not quoted arg, find next space (if any)
      i1, i2 = inputstr:find("%s+", i + 1)
      if i1 then
        --spaces after arg, more args to follow
        ret[retI] = inputstr:sub(i, i1 - 1)
        retI = retI + 1
        i = i2 + 1
      else
        --end of msg
        ret[retI] = inputstr:sub(i)
        return ret
      end
    end
  end
  
  return ret
end

SLASH_LOOKINGFORSATCHELS1 = "/lfs"
SlashCmdList["LOOKINGFORSATCHELS"] = function(msg, editbox)
  args = mysplit2(msg or "")
  
  if string.lower(args[1] or "") == "move" then
    if moving then
      moving = false
      frame:SetMovable(false)
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
      frame.bgtexture:Hide()
    else
      moving = true
      frame:SetMovable(true)
      frame:EnableMouse(true)
      frame:RegisterForDrag("LeftButton")
      frame.bgtexture:Show()
    end
    print("|cffaaaaffLookingForSatchels |rmove |cffaaaaffis now "..(moving == true and "|cffaaffaamoving" or "|cffff8888fixed"))
  elseif string.lower(args[1] or "") == "reset" then
    LookingForSatchels["framePoint"] = DefaultO["framePoint"]
    LookingForSatchels["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    LookingForSatchels["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    LookingForSatchels["frameOffsetX"] = DefaultO["frameOffsetX"]
    LookingForSatchels["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(LookingForSatchels["framePoint"], LookingForSatchels["frameRelativeTo"], LookingForSatchels["frameRelativePoint"], LookingForSatchels["frameOffsetX"], LookingForSatchels["frameOffsetY"])
    
    print("|cffaaaaffLookingForSatchels |rposition reset")
  elseif string.lower(args[1] or "") == "roles" then
    frame.LFG_roles[1] = tonumber(args[2])
    frame.LFG_roles[2] = tonumber(args[3])
    frame.LFG_roles[3] = tonumber(args[4])
    print("roles |cffaaaaffchange search for roles ("..(frame.LFG_roles[1]==1 and "|cffaaffaa" or "|cffff8888").."Tank "..(frame.LFG_roles[2]==1 and "|cffaaffaa" or "|cffff8888").."Heal "..(frame.LFG_roles[3]==1 and "|cffaaffaa" or "|cffff8888").."Damage|cffaaaaff)")
  else
    print("|cffaaaaffLookingForSatchels |r"..(GetAddOnMetadata("LookingForSatchels", "Version") or "").." |cffaaaaff(use |r/lfs <option> |cffaaaafffor these options)")
    print("  move |cffaaaafftoggle moving the frame ("..(moving == true and "|cffaaffaamoving" or "|cffff8888fixed").."|cffaaaaff)")
    print("  reset |cffaaaaffreset the frame's position")
    print("  roles |cffaaaaffchange search for roles ("..(frame.LFG_roles[1]==1 and "|cffaaffaa" or "|cffff8888").."Tank "..(frame.LFG_roles[2]==1 and "|cffaaffaa" or "|cffff8888").."Heal "..(frame.LFG_roles[3]==1 and "|cffaaffaa" or "|cffff8888").."Damage|cffaaaaff)")
  end
end
